"""
Module to process the log file generated from experiments to
get metrics such as accuracy
:author: gurnoorsingh(20221121)
"""

import re
import argparse
import random
import numpy as np
from collections import defaultdict
from constants import RELAY_REGIONS

parser = argparse.ArgumentParser()
parser.add_argument('-f', '--filename', help="Log file name", required=True)
args = parser.parse_args()

random.seed(42)
# samples to be used for subsampling
NUM_SAMPLES = 200



def calculate_metrics(predicted_orders, actual_senders_list):
    relay_node_metrics = defaultdict(lambda:0)
    relay_node_region_metrics = defaultdict(lambda:0)
    total_count = 0

    for predicted_order, actual_senders in zip(predicted_orders, actual_senders_list):
        predicted_region_order = []
        for item in predicted_order:
            region = RELAY_REGIONS[item]
            if region not in predicted_region_order:
                predicted_region_order.append(region)
        
        sender_regions = [RELAY_REGIONS[sender] for sender in actual_senders]

        for rank in range(len(predicted_order)):
                relay_node_metrics[rank] += (predicted_order[rank] in actual_senders)
            
        for rank in range(len(predicted_region_order)):
            relay_node_region_metrics[rank] += (predicted_region_order[rank] in sender_regions)

        # print(predicted_order, actual_senders)
        total_count += 1

    return relay_node_metrics, relay_node_region_metrics


def calculate_statistical_measures(relay_node_metrics_list):
    """
    Calculate mean and std of various ranks. The input could
    be relay node metrics or relay node region metrics
    """
    combined_metrics = defaultdict(lambda:[])
    for experiment_no in range(len(relay_node_metrics_list)):
        experiment_relay_node_metrics = relay_node_metrics_list[experiment_no]
        for rank in range(len(experiment_relay_node_metrics)):
            combined_metrics[rank].append(100 * experiment_relay_node_metrics[rank] / NUM_SAMPLES)
    
    for rank in range(len(combined_metrics)):
        print(rank, f"{np.mean(combined_metrics[rank]):.2f}", f"{np.std(combined_metrics[rank]):.2f}")



with open(args.filename) as fh:
    predicted_orders, actual_senders_list = [], []
 
    for line in fh.readlines():
        if "DEBUG - predicted:" in line:
            line_data = line.split('-', 2)
            predicted_order_dict = eval(re.findall("({.*?})", line_data[2])[0])
            predicted_order = list(predicted_order_dict.keys())
            actual_senders = eval(re.findall("(\[.*?\])", line_data[2])[0])
            
            predicted_orders.append(predicted_order)
            actual_senders_list.append(actual_senders)

    
    relay_node_metrics_list, relay_node_region_metrics_list = [], []

    for _ in range(25):
        indices = random.sample(range(len(predicted_orders)), k=NUM_SAMPLES)
        sampled_predicted_orders, sampled_actual_senders_list = [], []
        for idx in indices:
            sampled_predicted_orders.append(predicted_orders[idx])
            sampled_actual_senders_list.append(actual_senders_list[idx])
        

        relay_node_metrics, relay_node_region_metrics = calculate_metrics(sampled_predicted_orders, sampled_actual_senders_list)

        relay_node_metrics_list.append(relay_node_metrics)
        relay_node_region_metrics_list.append(relay_node_region_metrics)
    
    calculate_statistical_measures(relay_node_metrics_list)



            

            


